# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {
    "logger": "CLI_Version.ipynb",
    "Client": "Client.ipynb",
    "DataSource": "Client.ipynb",
    "DataSource.pull": "API_DataSource.ipynb",
    "DataSource.head": "API_DataSource.ipynb",
    "DataSource.train": "API_DataSource.ipynb",
    "DataSource.push": "API_DataSource.ipynb",
    "ensure_is_instance": "API_Helper.ipynb",
    "get_base_url": "API_Helper.ipynb",
    "post_data": "API_Helper.ipynb",
    "get_data": "API_Helper.ipynb",
    "delete_data": "API_Helper.ipynb",
    "add_ready_column": "API_Helper.ipynb",
    "generate_df": "API_Helper.ipynb",
    "get_tag_str": "API_Helper.ipynb",
    "APIKey": "Client.ipynb",
    "Model": "Client.ipynb",
    "Model.evaluate": "API_Model.ipynb",
    "Model.predict": "API_Model.ipynb",
    "Prediction": "Client.ipynb",
    "Prediction.to_pandas": "API_Prediction.ipynb",
    "Prediction.push": "API_Prediction.ipynb",
    "ProgressStatus": "Client.ipynb",
    "ProgressStatus.wait": "API_ProgressStatus.ipynb",
    "User": "Client.ipynb",
    "activated": "ActivateTestingEnvironment.ipynb",
    "app": "CLI_Version.ipynb",
    "local_ds_app": "CLI_DataSource.ipynb",
    "s3": "CLI_DataSource.ipynb",
    "db": "CLI_DataSource.ipynb",
    "ls": "CLI_User.ipynb",
    "dtypes": "CLI_DataSource.ipynb",
    "head": "CLI_DataSource.ipynb",
    "train": "CLI_DataSource.ipynb",
    "details": "CLI_Pred.ipynb",
    "rm": "CLI_Pred.ipynb",
    "push": "CLI_Pred.ipynb",
    "tag": "CLI_DataSource.ipynb",
    "csv": "CLI_DataSource.ipynb",
    "authenicate_user": "CLI_Helper.ipynb",
    "requires_auth_token": "CLI_Helper.ipynb",
    "humanize_date": "CLI_Helper.ipynb",
    "humanize_number": "CLI_Helper.ipynb",
    "humanize_size": "CLI_Helper.ipynb",
    "create": "CLI_User.ipynb",
    "predict": "CLI_Model.ipynb",
    "evaluate": "CLI_Model.ipynb",
    "to_pandas": "CLI_Pred.ipynb",
    "token": "CLI_Token.ipynb",
    "disable": "CLI_User.ipynb",
    "enable": "CLI_User.ipynb",
    "update": "CLI_User.ipynb",
    "version": "CLI_Version.ipynb",
    "logging_level": "Logger.ipynb",
    "CLILoggerHandler": "Logger.ipynb",
    "handler": "Logger.ipynb",
    "supress_timestamps": "Logger.ipynb",
    "get_logger": "Logger.ipynb",
    "should_supress_timestamps": "Logger.ipynb",
    "set_level": "Logger.ipynb",
}

modules = [
    "components/client.py",
    "components/datasource.py",
    "helper.py",
    "components/api_key.py",
    "components/model.py",
    "components/prediction.py",
    "components/progress_status.py",
    "components/user.py",
    "testing/activate_by_import.py",
    "cli/cli.py",
    "cli/ds.py",
    "cli/helper.py",
    "cli/api_key.py",
    "cli/model.py",
    "cli/pred.py",
    "cli/token.py",
    "cli/user.py",
    "cli/version.py",
    "client.py",
    "logger.py",
]

doc_url = "//"

git_url = "https://gitlab.com/airt.ai/airt-client/tree/main/"


def custom_doc_links(name):
    return None
