# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_Version.ipynb (unless otherwise specified).

__all__ = ["logger"]

# Cell

from typing import *

# Internal Cell

import typer
import logging
from tabulate import tabulate

from ..client import Client
from ..logger import get_logger, set_level

# Internal Cell

app = typer.Typer()

# Cell

logger = get_logger(__name__)

# Internal Cell


def version() -> None:
    """Get the version of airt client and airt service."""
    df = Client.version().to_frame()

    typer.echo(tabulate(df, headers="keys", tablefmt="plain"))


# Internal Cell


app.command()(version)
