# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/ActivateTestingEnvironment.ipynb (unless otherwise specified).

__all__ = ["activated"]

# Cell


def activated() -> bool:
    return True


# Internal Cell

from ..logger import get_logger, set_level, supress_timestamps  # type: ignore

supress_timestamps()

# Cell

import logging

# Internal Cell

logger = get_logger(__name__)

set_level(logging.INFO)

logger.info("Testing environment activated.")
